/*
 * (c) 2005 Sebastian Zagrodzki <s.zagrodzki@net.icm.edu.pl>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <string.h>
#include <ctype.h>


#include "id.h"
#include "common.h"

int id_size = 16;

char *losuj_id() {
	char *wynik, *p;
	int i, j;
	struct timeval tv;

	wynik = calloc(id_size + 1, sizeof(char));
	if (!wynik) { perror("calloc"); abort(); }
	p = wynik;

	if (gettimeofday(&tv, NULL) < 0) { perror("gettimeofday"); abort(); }
	srand(tv.tv_usec);
	for (i = 0; i < id_size; i++) {
		j = (int)(16.0 * rand()/(RAND_MAX+1.0));
		sprintf(p++, "%x", j);
	}
	return wynik;
}
	
char *get_backup_id(char *arg) {
	char *ret, *spac, *tag;
	tag = (char*)strdup(arg);
	if (!tag) { perror("strdup"); abort(); }
	if (strncmp("BACKUP ", tag, 7))
		error("ID read has incorrect content");
	tag += 7;
	spac = (char*)strchr(tag, ' ');
	if (!spac)
		error("ID read has incorrect content");
	*spac = '\0';
	ret = (char*)strdup(tag);
	if (!ret) { perror("strdup"); abort(); }
	*spac = ' ';
	debug(1, "Found backup ID: %s", ret);
	return ret;
}
		
int set_backup_id(char *tag) {
	char *noweid;
	int len;
	noweid = NULL;
	asprintf(&noweid, "BACKUP %s TAPE %d", id, tapeno);
	debug(1, "Generated ID tag: %s", noweid);
	len = strlen(noweid);
	if (len > blocksize-1) error("strlen(ID) > blocksize");
	strcpy(tag, noweid);
	return len;
}

int set_backup_end(char *tag) {
	char *noweid;
	int len;
	noweid = NULL;
	asprintf(&noweid, "BACKUP %s EOD", id);
	debug(1, "Generated end tag: %s", noweid);
	len = strlen(noweid);
	if (len > blocksize-1) error("strlen(EOD) > blocksize");
	strcpy(tag, noweid);
	return len;
}

void check_backup_id(char *tag, int len) {
	char *oczekiwany;

	asprintf(&oczekiwany, "BACKUP %s TAPE %d", id, tapeno);
	debug(1, "check_backup_id(): expected ID tag: %s", oczekiwany);
	if (strlen(oczekiwany) != len) {
		error_msg("Expected ID: BACKUP %s TAPE %d", id, tapeno);
		error("ID read has different length than expected");
	}
	if (strncmp(oczekiwany, tag, len)) {
		char *i;
		for (i = tag; *i; i++)
			if (!isalnum(*i)&&!isspace(*i)) {
				error_msg("Expected ID: BACKUP %s TAPE %d",
					id, tapeno);
				error("ID read contains nonprintable characters");
			}
		error_msg("Expected ID: BACKUP %s TAPE %d", id, tapeno);
		error("ID read: %s", tag);
	}	
	debug(1, "check_backup_id(): OK");
}

void check_backup_end(char *tag, int len) {
	char *oczekiwany;

	asprintf(&oczekiwany, "BACKUP %s EOD", id);
	debug(1, "check_backup_end(): Expected end tag: %s", oczekiwany);
	if (strlen(oczekiwany) != len) {
		error_msg("Expected ID tag: BACKUP %s TAPE %d", id, tapeno);
		error("ID read has different length than expected");
	}
	if (strncmp(oczekiwany, tag, len)) {
		char *i;
		for (i = tag; *i; i++)
			if (!isalnum(*i)&&!isspace(*i)) {
				error_msg("Expected ID: BACKUP %s EOD", id);
				error("ID read contains nonprintable characters");
			}
		error_msg("Expected ID: BACKUP %s EOD", id);
		error("ID read: %s", tag);
	}	
	debug(1, "check_backup_id(): OK");
}
