/*
 * (c) 2005 Sebastian Zagrodzki <s.zagrodzki@net.icm.edu.pl>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <string.h>
#include <sys/mtio.h>
#include <stdlib.h>
#include "common.h"

void check_drive_mode() {
	struct mtget mtstatus;
	if (ioctl(tape_fd, MTIOCGET, &mtstatus) < 0) {
		perror("ioctl"); abort();
	}
	drive_mode = (mtstatus.mt_dsreg & MT_ST_BLKSIZE_MASK) >> MT_ST_BLKSIZE_SHIFT;
	debug(1, "Drive block size: %d", drive_mode);
	if ((blocksize == 0) && (drive_mode == 0)) {
		blocksize = (mode == MODE_READ ? defvarreadblocksize : defvarwriteblocksize);
		notice("Setting default %s block size: %d",
				(mode == MODE_READ ? "read" : "write"),
				blocksize);
		warning("Default blocksize is probably much larger than necessary.");
		warning("This can result in much decreased performance");
	} else if (blocksize == 0) {
		notice("Setting block size based on drive settings: %d", drive_mode);
		blocksize = drive_mode;
	} else if ((drive_mode > 0) && (blocksize > 0)) 
		warning("blocksize (%d) != drive blocksize (%d)",
			blocksize, drive_mode);
}

void write_eof() {
	struct mtop mt_cmd;
	mt_cmd.mt_op = MTWEOF;
	mt_cmd.mt_count = 1;
	if (ioctl(tape_fd, MTIOCTOP, &mt_cmd) < 0) {
		perror("ioctl"); abort();
	}
}

