/*
 * (c) 2005 Sebastian Zagrodzki <s.zagrodzki@net.icm.edu.pl>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "change_tape.h"
#include "readwrite.h"
#include "id.h"
#include "mode.h"

void do_read() {
	char *tagbuf = NULL;
	block_stat b;

	while (1) {
		open_tape_dev();
		check_drive_mode();
		if (tag_tapes) {
			int x;
			if (!tagbuf) tagbuf = (char *)malloc(blocksize);
			b = read_block(tagbuf, 1);
			if (b.len == 0)
				error("Found EOF - no ID found");
			if (!id) id = get_backup_id(tagbuf);
			check_backup_id(tagbuf, b.len);
			while (1) {
				x = 1;
				b = read_block(tagbuf, 1);
				if (b.len > 0) x++;
				else {
					if (x > 1) warning("Too small blocksize - opening tag has %d blocks instead of 1.", x);
					break;
				}
			}
		}
		read_tape();
		debug(2, "Expecting end tag or end of tape");
		b = read_block(tagbuf, 1);
		if ((b.len > 0) && tag_end) {
			debug(2, "1 block read. Is it ending block?");
			check_backup_end(tagbuf, b.len);
			debug(1, "Matching end tag found.");
			koniec_backupu();
		} else if (b.len > 0) {
			// heurystyka - jeli nie tagujemy kocw backupu,
			// a skoczy nam si plik i jest za nim dalej co zapisane
			// to to jest koniec backupu
			debug(1, "There is some other data written after EOF, and we don't expect the end tag");
			koniec_backupu();
		}
		// ret == 0 - znaczy e EOD (podwjny read = 0)
		if (max_tapes && (tapeno >= max_tapes)) {
			debug(1, "Tape number limit reached");
			koniec_backupu();
		}
		close_tape_dev();
		tape_stats();
		tapeno++;
		if (change_tape()) {
			debug(1, "change_tape() returned \"OK, don't read next tape\"");
			koniec_backupu();
		}
	}
}

void do_write() {
	char *tagbuf = NULL;
	int taglen;
	char *blockbuf = NULL;
	int r;

	if (!tagbuf) tagbuf = (char *)malloc(blocksize);
	if (!tagbuf) { perror("malloc"); abort(); }
	
	while (1) {
		open_tape_dev();
		check_drive_mode();
		if (tag_tapes) {
			debug(1, "Writing tape ID tag");
			if (!id) id = (char*)strdup(losuj_id());
			taglen = set_backup_id(tagbuf);
			r = write_block(tagbuf, taglen);
			if (!r)
				error("Couldn't write beginning ID tag on new tape");
			write_eof();
		}
		blockbuf = write_tape(blockbuf);
		if (blockbuf == NULL) {
			write_eof();
			// koniec danych do zapisania
			if (tag_end) {
				taglen = set_backup_end(tagbuf);
				r = write_block(tagbuf, taglen);
				if (!r) {
					char *tagbuftemp;
					int taglentemp;
					warning("Backup was completed, but end tag couldn't be written (no space left?)");
					tapeno++;
					if (max_tapes && (tapeno > max_tapes))
						error("Tape number limit reached");
					close_tape_dev();
					tape_stats();
					change_tape();
					open_tape_dev();
					if (!tagbuftemp) tagbuftemp = (char *)malloc(blocksize);
					if (!tagbuftemp) { perror("malloc"); abort(); }
					debug(1, "Writing tape ID tag");
					taglentemp = set_backup_id(tagbuftemp);
					r = write_block(tagbuftemp, taglentemp);
					if (!r)
						error("Couldn't write beginning ID tag on the new tape");
					free(tagbuftemp);
					write_eof();
					r = write_block(tagbuf, taglen);
					if (!r)
						error("Couldn't write END tag directly after beginning ID tag");
				}
			}
			close_tape_dev();
			koniec_backupu();
		}
		close_tape_dev();
		if (max_tapes && (tapeno >= max_tapes)) {
			debug(1, "Tape number limit reached");
			koniec_backupu();
		}
		tape_stats();
		tapeno++;
		change_tape();
	}
}
